{
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
PLATFORM  : Windows 9x/NT/XP - Delphi 5,6,7
PROJECT   : eXpress Application Manager
FILE      : F_XAM_Exception.pas
PURPOSE   : Exception informations box.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
LICENSE   : LGPL
AUTHOR    : Copyright 2003 Olivier Rogier 
WEBSITE   : http://www.ordisoftware.info
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CREATION  : v 1.0.0 - 01/2003
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
}

unit F_XAM_Exception;

{
#######################################################################################################################
                                                       INTERFACE
#######################################################################################################################
}

interface

uses
  SysUtils, Forms, StdCtrls, Buttons, Controls, Classes, ExtCtrls, Graphics;
  
type
  TfmXAMException = class(TForm)
    gbExec           : TGroupBox;
    gbSys            : TGroupBox;
    pnDetail         : TPanel;
    pnInfos          : TPanel;
    pnInfosSystem    : TPanel;
    pnInfosExecution : TPanel;
    pnButtonsDetail  : TPanel;
    pnMain           : TPanel;
    pnButtons        : TPanel;
    pnMessage        : TPanel;
    pnSeparator      : TPanel;
    lbAppVer         : TLabel;
    lbAppPath        : TLabel;
    lbSenderClass    : TLabel;
    lbClass          : TLabel;
    lbProc           : TLabel;
    lbAddr           : TLabel;
    lbLine           : TLabel;
    lbSenderName     : TLabel;
    lbUnit           : TLabel;
    edAppVer         : TEdit;
    edUnit           : TEdit;
    edLine           : TEdit;
    edSenderClass    : TEdit;
    edAddr           : TEdit;
    edProc           : TEdit;
    edClass          : TEdit;
    edSenderName     : TEdit;
    btnPrint         : TButton;
    btnLog           : TButton;
    btnClose         : TButton;
    btnDetail        : TButton;
    edAppPath        : TEdit;
    Panel2           : TPanel;
    Panel1           : TPanel;
    Panel3           : TPanel;
    moMessage        : TMemo;
    pnMessageTitle   : TPanel;
    pnException      : TPanel;
    Image1           : TImage;
    btnCallstack     : TButton;
    procedure btnPrintClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure btnLogClick(Sender: TObject);
    procedure btnDetailClick(Sender: TObject);
    procedure OnKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure btnCallstackClick(Sender: TObject);
  private
    E : Exception;
    S : TObject;
  public
    class procedure Execute(Sender : TObject; ExceptObject : Exception);
  end;

{
#######################################################################################################################
                                                     IMPLEMENTATION
#######################################################################################################################
}

implementation

uses
  Dialogs, Windows, U_XAM_System, U_XAM_Global, U_XAM_Debug, U_XAM_LogFile;

{$R *.DFM}

const
  CAPTION_DETAILS = 'Details';

//-------------------------------------------------------------------------------------------------
// TfmXAMException.Execute
//-------------------------------------------------------------------------------------------------
class procedure TfmXAMException.Execute(Sender : TObject; ExceptObject : Exception);
begin
  with (XAMDebugger), GetExceptionInfos(Sender, ExceptObject), Create(nil) do
    try
      Caption               := Application.Title;
      E                     := ExceptObject;
      S                     := Sender;
      btnDetail.Visible     := GetMapFile.Present;
      btnLog.Enabled        := XAMApplication.LogFile.Enabled;
      moMessage.Text        := EMessage;
      pnException.Caption   := ' ' + EFullName;
      edAppPath.Text        := AppPath;
      edAppVer.Text         := AppVer;
      edUnit.Text           := CodeUnit;
      edLine.Text           := CodeLine;
      edSenderClass.Text    := SenderClass;
      edSenderName.Text     := SenderName;
      edClass.Text          := CodeClass;
      edProc.Text           := CodeProc;
      edAddr.Text           := EAddress;
      ShowModal;
    finally
      Free;
    end;
end;

//-------------------------------------------------------------------------------------------------
// TfmXAMException.FormCreate
//-------------------------------------------------------------------------------------------------
procedure TfmXAMException.FormCreate(Sender: TObject);
begin
  Caption := XAMApplication.Title;
  btnDetail.Click;
  Application.BringToFront;
end;

//-------------------------------------------------------------------------------------------------
// TfmXAMException.KeyDown
//-------------------------------------------------------------------------------------------------
procedure TfmXAMException.OnKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  if (Key = VK_ESCAPE) then btnClose.Click;
end;

//-------------------------------------------------------------------------------------------------
// TfmXAMException.btnDetailClick
//-------------------------------------------------------------------------------------------------
procedure TfmXAMException.btnDetailClick(Sender: TObject);
begin
  case (pnDetail.Visible) of
    True  : begin
              pnDetail.Visible := false;
              pnDetail.Caption := CAPTION_DETAILS + ' >>';
              Height           := Height - pnDetail.Height;
            end;
    False : begin
              Height           := Height + pnDetail.Height;
              pnDetail.Caption := CAPTION_DETAILS + ' <<';
              pnDetail.Visible := true;
            end;
  end;
end;

//-------------------------------------------------------------------------------------------------
// TfmXAMException.btnLogClick
//-------------------------------------------------------------------------------------------------
procedure TfmXAMException.btnLogClick(Sender: TObject);
begin
  XAMApplication.LogFile.Show;
end;

//-------------------------------------------------------------------------------------------------
// TfmXAMException.btnPrintClick
//-------------------------------------------------------------------------------------------------
procedure TfmXAMException.btnPrintClick(Sender: TObject);
begin
  Print;
end;

//-------------------------------------------------------------------------------------------------
// TfmXAMException.btnCallstackClick
//-------------------------------------------------------------------------------------------------
procedure TfmXAMException.btnCallstackClick(Sender: TObject);
begin
  ShowMessage(XAMDebugger.GetCallStack);
end;

end.

