{
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
PLATFORM  : Windows 9x/NT/XP - Delphi 5,6,7
PROJECT   : eXpress Application Manager
FILE      : U_XAM_AutoSplash.pas
PURPOSE   : Application splash screen with auto form/datamodule creation.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
LICENSE   : LGPL
AUTHOR    : Copyright 2003 Olivier Rogier 
WEBSITE   : http://www.ordisoftware.info
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CREATION  : v 1.0.0 - 01/2003
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
USAGE     : Replace the Delphi standard form creation (Application.CreateForm) by
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
}

unit U_XAM_AutoSplash;

{
#######################################################################################################################
                                                       INTERFACE
#######################################################################################################################
}

interface

uses
  Classes, Forms;

type
  TXAMOnSplashStart    = procedure (Sender : TObject; StepCount : integer) of object;
  TXAMOnSplashProgress = procedure (Sender : TObject; Step : integer; Caption : string) of object;

  TXAMAutoSplash = class
  private
    FList       : TCollection;
    FForm       : TForm;
    FOnStart    : TXAMOnSplashStart;
    FOnProgress : TXAMOnSplashProgress;
  public
    constructor Create; overload;
    constructor Create(AForm : TForm; AOnStart : TXAMOnSplashStart; AOnProgress : TXAMOnSplashProgress); overload;
    destructor  Destroy; override;
    procedure   Add(Caption : string; ADelay : integer = 0); overload;
    procedure   Add(Caption : string; FormClass: TComponentClass; var Ref; ADelay : integer = 0); overload;
    procedure   Execute;
  public
    property Form       : TForm                read FForm       write FForm;
    property OnStart    : TXAMOnSplashStart    read FOnStart    write FOnStart;
    property OnProgress : TXAMOnSplashProgress read FOnProgress write FOnProgress;
  end;

{
#######################################################################################################################
                                                     IMPLEMENTATION
#######################################################################################################################
}

implementation

uses
  Windows;

type
  TFormPtr = ^TForm;
  TFormAllocItem = class(TCollectionItem)
  public
    Text          : string;
    InstanceClass : TComponentClass;
    Reference     : TFormPtr;
    Delay         : integer;
  end;

//-------------------------------------------------------------------------------------------------
// TXAMAutoSplash.Create
//-------------------------------------------------------------------------------------------------
constructor TXAMAutoSplash.Create;
begin
  FList := TCollection.Create(TFormAllocItem);
end;

constructor TXAMAutoSplash.Create(AForm: TForm; AOnStart: TXAMOnSplashStart; AOnProgress: TXAMOnSplashProgress);
begin
  Create;
  FForm       := AForm;
  FOnStart    := AOnStart;
  FOnProgress := AOnProgress;
end;

//-------------------------------------------------------------------------------------------------
// TXAMAutoSplash.Destroy
//-------------------------------------------------------------------------------------------------
destructor TXAMAutoSplash.Destroy;
begin
  FList.Free;
  inherited;
end;

//-------------------------------------------------------------------------------------------------
// TXAMAutoSplash.Add
//-------------------------------------------------------------------------------------------------
// PURPOSE : Add a form to the creation list.
// IN      : Caption     -> Title or '' if name class used
//           FormClass   -> Class
//           Ref         -> Reference
//           Delay       -> Pause
//-------------------------------------------------------------------------------------------------
procedure TXAMAutoSplash.Add(Caption: string; ADelay: integer);
begin
  with (FList.Add as TFormAllocItem) do
    begin
      Text          := Caption;
      InstanceClass := nil;
      Reference     := nil;
      Delay         := ADelay;
    end;
end;

procedure TXAMAutoSplash.Add(Caption : string; FormClass: TComponentClass; var Ref; ADelay : integer = 0);
begin
  with (FList.Add as TFormAllocItem) do
    begin
      case (Caption = '') of
        True  : Text := FormClass.ClassName;
        False : Text := Caption;
      end;
      InstanceClass := FormClass;
      Reference     := @TForm(Ref);
      Delay         := ADelay;
    end;
end;

//-------------------------------------------------------------------------------------------------
// TXAMAutoSplash.Execute
//-------------------------------------------------------------------------------------------------
procedure TXAMAutoSplash.Execute;
var i : integer;
begin
  if Assigned(FOnStart) then FOnStart(self, FList.Count);
  if Assigned(FForm)    then FForm.Show;
  try
    for i := 0 to Pred(FList.Count) do
      with (FList.Items[i] as TFormAllocItem) do
        begin
          if Assigned(FOnProgress) then FOnProgress(self, i, Text);
          if Assigned(FForm) then FForm.Refresh;
          Sleep(Delay);
          if (not Application.Terminated) then
            if Assigned(InstanceClass) and Assigned(Reference) then
              Application.CreateForm(InstanceClass, Reference^);
        end;
  finally
    if Assigned(FForm) then FForm.Hide;
  end;
end;

end.

