{
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
PLATFORM  : Windows 9x/NT/XP - Delphi 5,6,7
PROJECT   : eXpress Application Manager
FILE      : U_XAM_FileVersion.pas
PURPOSE   : File version informations.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
LICENSE   : LGPL
AUTHOR    : Copyright 2003 Olivier Rogier 
WEBSITE   : http://www.ordisoftware.info
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CREATION  : v 1.0.0 - 01/2003
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
}

unit U_XAM_FileVersion;

{
#######################################################################################################################
                                                       INTERFACE
#######################################################################################################################
}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms;

type
  TXAMFileVersion = class
  private
    FFilename         : String;
    FMajor            : string;
    FMinor            : string;
    FRelease          : String;
    FBuild            : String;
    FVersion          : String;
    FProductVersion   : String;
    FOSName           : String;
    FFileType         : String;
    FComments         : String;
    FCompanyName      : String;
    FFileDescription  : String;
    FInternalName     : String;
    FLegalCopyright   : String;
    FLegalTradeMarks  : String;
    FOriginalFileName : String;
    FPrivateBuild     : String;
    FProductName      : String;
    FSpecialBuild     : String;
  private
    procedure Init;
    procedure SetFilename(AFileName : string);
  public
    property Filename         : String read FFilename write SetFilename;
    property Major            : string read FMajor;
    property Minor            : string read FMinor;
    property Release          : string read FRelease;
    property Build            : string read FBuild;
    property Version          : string read FVersion;
    property ProductVersion   : string read FProductVersion;
    property OSName           : string read FOSName;
    property FileType         : String read FFileType;
    property Comments         : String read FComments;
    property CompanyName      : String read FCompanyName;
    property FileDescription  : string read FFileDescription;
    property InternalName     : string read FInternalName;
    property LegalCopyright   : String read FLegalCopyright;
    property LegalTradeMarks  : String read FLegalTradeMarks;
    property OriginalFileName : string read FOriginalFileName;
    property PrivateBuild     : String read FPrivateBuild;
    property ProductName      : String read FProductName;
    property SpecialBuild     : string read FSpecialBuild;
  end;

function XAMGetFileVersion(AFileName : string) : string;

{
#######################################################################################################################
                                                     IMPLEMENTATION
#######################################################################################################################
}

implementation

uses
  U_XAM_Global;

type
  TOsIndex       = ( VOS_DOS, VOS_OS216, VOS_OS232, VOS_NT, VOS__WINDOWS16, VOS__PM16, VOS__PM32, VOS__WINDOWS32,
                     VOS_DOS_WINDOWS16, VOS_DOS_WINDOWS32, VOS_OS216_PM16, VOS_OS232_PM32, VOS_NT_WINDOWS32 );

  TFileTypeIndex = ( VFT_APP, VFT_DLL, VFT_DRV, VFT_FONT, VFT_VXD, VFT_STATIC_LIB );

const
  OS_NAMES : array[TOsIndex] of string
           = ( 'MS-DOS', '16-bit OS/2', '32-bit OS/2', 'Windows NT', 'Windows 3.xx', '16-bit OS/2 Presentation Manager',
               '32-bit OS/2 Presentation Manager', '32-bit Windows', 'Windows 3.xx with MS-DOS', '32-bit Windows with MS-DOS',
               '16-bit OS/2 with Presentation Manager', '32-bit OS/2 with Presentation Manager', 'Windows NT with 32 bit Windows' );

  FILETYPE_NAMES : array[TFileTypeIndex] of string
                 = ( 'Application', 'Dynamic-Link Library', 'Device Driver', 'Font', 'Virtual Device Driver', 'Static-Link Library' );

//-------------------------------------------------------------------------------------------------
// XAMGetFileVersion
//-------------------------------------------------------------------------------------------------
function XAMGetFileVersion(AFileName : string) : string;
begin
  result := '0.0.0';
  try
    with (TXAMFileVersion.Create) do
      try
        Filename := AFileName;
        result   := Major + '.' + Minor + '.' + Release;
      finally
        Free;
      end;
  except
    on E : Exception do result := E.Message;
  end;
end;

//-------------------------------------------------------------------------------------------------
// TXAMFileVersion.SetFilename
//-------------------------------------------------------------------------------------------------
procedure TXAMFileVersion.SetFilename(AFileName : String);
begin
  if (FFilename = AFileName) then Exit;
  if not FileExists(AFileName) then raise EInOutError.CreateFmt(XAM_ERR_FILE_INVALID, [AFileName]);
  FFilename := AFileName;
  Init;
end;

//-------------------------------------------------------------------------------------------------
// TXAMFileVersion.Init
//-------------------------------------------------------------------------------------------------
procedure TXAMFileVersion.Init;
var h, ln       : cardinal;
    nb          : integer;
    s           : string;
    rec         : PVSFixedFileInfo;
    pVer, pItem : pointer;
    procedure InitVar(Section : string; var Value : string);
    begin
      if VerQueryValue(pVer, PChar(s + Section), pItem, ln) then Value := PChar(pItem);
    end;
begin
  try
    nb := GetFileVersionInfoSize(PChar(FFilename), h);
    if (nb = 0) then Exit;
    GetMem(pVer, nb);
    try
      if GetFileVersionInfo(PChar(FFilename), h, nb, pVer) then
        begin
          if not VerQueryValue(pVer, '\', Pointer(rec), ln) then Abort;
          with (rec^) do
            begin
              FMajor          := IntToStr(HIWORD(dwFileVersionMS));
              FMinor          := IntToStr(LOWORD(dwFileVersionMS));
              FRelease        := IntToStr(HIWORD(dwFileVersionLS));
              FBuild          := IntToStr(LOWORD(dwFileVersionLS));
              FVersion        := Format('%s.%s.%s.%s', [FMajor, FMinor, FRelease, FBuild]);
              FProductVersion := Format('%d.%d.%d',    [HIWORD(dwProductVersionMS), LOWORD(dwProductVersionMS),
                                                       HIWORD(dwProductVersionLS), LOWORD(dwProductVersionLS)]);
            case (TOsIndex(dwFileOS) in [Low(TOsIndex), High(TOsIndex)]) of
              True  : FOSName := OS_NAMES[TOsIndex(dwFileOS)];
              False : FOSName := XAM_CAPTION_UNKNOWN;
            end;
            case (TFileTypeIndex(dwFileType) in [Low(TFileTypeIndex), High(TFileTypeIndex)]) of
              True  : FFileType := FILETYPE_NAMES[TFileTypeIndex(dwFileType)];
              False : FFileType := XAM_CAPTION_UNKNOWN;
            end;
           end;
          s := '\StringFileInfo\' + IntToHex($409, 4) + IntToHex(1252, 4) + '\';
          InitVar('Comments',         FComments);         InitVar('CompanyName',      FCompanyName);
          InitVar('FileDescription',  FFileDescription);  InitVar('InternalName',     FInternalName);
          InitVar('LegalCopyright',   FLegalCopyright);   InitVar('LegalTradeMarks',  FLegalTradeMarks);
          InitVar('OriginalFileName', FOriginalFileName); InitVar('PrivateBuild',     FPrivateBuild);
          InitVar('ProductName',      FProductName);      InitVar('SpecialBuild',     FSpecialBuild);
        end;
    finally
      FreeMem(pVer, nb);
    end;
  except
    raise EInOutError.CreateFmt(XAM_ERR_GET_FILE_INFOS, [FFileName]);
  end;
end;

end.

