{
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
PLATFORM  : Windows 9x/NT/XP - Delphi 5,6,7
PROJECT   : eXpress Application Manager
FILE      : U_XAM_Global.pas
PURPOSE   : Global constants, datas & functions.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
LICENSE   : LGPL
AUTHOR    : Copyright 2003 Olivier Rogier 
WEBSITE   : http://www.ordisoftware.info
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CREATION  : v 1.0.0 - 01/2003
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
}

unit U_XAM_Global;

{
#######################################################################################################################
                                                       INTERFACE
#######################################################################################################################
}

interface

uses
  Windows, Registry;

{
<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<->>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
                                                       Constants
<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<->>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
}

const
  XAM_VCL_PAGE            = 'XAM';
  XAM_PROJECT_CODE        = 'XAM';
  XAM_PROJECT_NAME        = 'eXpress Application Manager';

  XAM_CAPTION_UNKNOWN     = '<Unknown>';
  XAM_CAPTION_NA          = '<N/A>';

  XAM_ERR_INVALID_INDEX   = 'Invalid index < %s = %d > doit tre entre [ %d .. %d ] !';
  XAM_ERR_FILE_INVALID    = 'Invalid file : %s !';
  XAM_ERR_GET_FILE_INFOS  = 'Error on getting file informations : %s';

  ENDL                    = #13#10;
  ENDL2                   = ENDL + ENDL;

var
  XAM_DEBUG   : boolean = false;
  XAM_LOGFILE : boolean = false;
{
<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<->>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
                                                       Functions
<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<->>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
}

// System
procedure MessageAbort(Msg : string; DoTerminate : boolean = false; DoHardTerminate : boolean = false);
function  FindFormAndShow(FormClassName, FormCaption : string) : boolean;
function  FormatText(Text : string; FirstLine, Corpus : integer; Car : Char = ' ') : string;
function  CaseString(Text : string; List : array of string) : integer;
function  CaseObject(Obj : TObject; List : array of TObject) : integer;
procedure FreeObject(var Obj);
function  VarToDisp(Value : Variant) : IDispatch;
function  VarToText(Value : variant) : string;
function  VarRecToText(VarRec : TVarRec) : string;

// Registry & Inifiles
function  RegOpenKey    (Reg : TRegistry; Key : String; CanCreate : boolean= true; DoRaise : boolean = true) : boolean;
function  RegReadInteger(Reg : TRegistry; Name : String; Default : integer) : integer;
function  RegReadBool   (Reg : TRegistry; Name : String; Default : boolean) : boolean;
function  RegReadString (Reg : TRegistry; Name : String; Default : string)  : string;

// SystemFile
procedure CreateEmptyFile(s : string);
procedure CheckFileExists(FileName : string);
function  IsFileInUse(FileName : string ) : boolean;
function  ExecuteFile(FileName : string; Shell : boolean = false; Params : string = ''; DefaultDir : string = ''; ShowCmd : word = SW_SHOW) : THandle;
function  WinExec32(FileName : string; ShowCmd : word; Wait : boolean) : dword;
function  GetLongFileName(FileName : string) : string;
function  GetFileSize(FileName : string) : integer;
procedure DoCopyFile(Src, Dest : string);
procedure DoMoveFile(Src, Dest : string);
procedure DoDeleteFile(Src : string);

{
<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<->>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
                                                        Classes
<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<->>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
}

type
  // **********************************************************************************************
  // TXAMObjectInfos
  // **********************************************************************************************
  // PURPOSE : Get object informations
  // **********************************************************************************************
  TXAMObjectInfos = class
  private
    FTarget          : TObject;       // Objet
    FComponentName   : string;        // Name if TComponent
    FTargetClassName : string;        // Class
    FParentClassName : string;        // Parent class
    FText            : string;        // Short description
    FInherits        : string;        // Parents class list
    FOwners          : string;        // Owners class list if TComponent
  private
    procedure SetTarget(Value : TObject);
  public
    class function GetObjectInfos(AObject : TObject) : string;
  public
    property Target          : TObject read FTarget           write SetTarget;
    property TargetClassName : string  read FTargetClassName;
    property ParentClassName : string  read FParentClassName;
    property Text            : string  read FText;
    property Inherits        : string  read FInherits;
    property Owners          : string  read FOwners;
    property ComponentName   : string  read FComponentName;
  end;


{
#######################################################################################################################
                                                     IMPLEMENTATION
#######################################################################################################################
}

implementation

uses
  {$ifdef VER140} StrUtils, Variants, {$endif} 
  {$ifdef VER150} StrUtils, Variants, {$endif}
  ShellApi, Classes, Forms, SysUtils, U_XAM_System, U_XAM_LogFile;

const
  XAMERR_FILE_NOT_FOUND = 'File not found : %s';
  XAMERR_ACCESS_REG     = 'Registry access error :' + ENDL + 'Cl < %s >';

{
<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<->>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
                                                         System
<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<->>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
}

//-------------------------------------------------------------------------------------------------
// MessageAbort
//-------------------------------------------------------------------------------------------------
// PURPOSE  : Show an error message, write it to log and break the execution.
// IN       : Msg             -> Message
//            DoTerminate     -> True if the application is stopped, false if not
//            DoHardTerminate -> True if the application is hard terminated, false if softly
// EXCEPT   : EAbort
//-------------------------------------------------------------------------------------------------
procedure MessageAbort(Msg : string; DoTerminate : boolean = false; DoHardTerminate : boolean = false);
begin
  if Assigned(XAMApplication) then XAMApplication.LogFile.Write(logError, Msg);
  Application.BringToFront;
  Application.MessageBox(PChar(Msg), PChar(Application.Title), MB_OK or MB_ICONERROR);
  if (DoTerminate) then
    case (DoHardTerminate) of
      True  : Halt;
      False : Application.Terminate;
    end;
  Abort;
end;

//-------------------------------------------------------------------------------------------------
// FindFormAndShow
//-------------------------------------------------------------------------------------------------
// PURPOSE  : Search a screen form and show it.
// IN       : FormClassName -> Form class name
//            FormCaption   -> Window caption
// OUT      : Return trie if found, false if not
//-------------------------------------------------------------------------------------------------
function FindFormAndShow(FormClassName, FormCaption : string) : boolean;
var i : integer;
begin
  result := false;
  for i := 0 to Pred(Screen.FormCount) do
    with (Screen.Forms[i]) do
      if (ClassName = FormClassName) and (Caption = FormCaption) then
        begin
          Show;
          result := true;
          Exit;
        end;
end;

//-------------------------------------------------------------------------------------------------
// FormatText
//-------------------------------------------------------------------------------------------------
// PURPOSE  : Format a text with first line and corpus margin.
// IN       : Text   -> Text
//            First  -> First line margin
//            Corpus -> Corpus margin
//            Car    -> Filling char
// OUT      : Return the formated text
//-------------------------------------------------------------------------------------------------
function FormatText(Text : string; FirstLine, Corpus : integer; Car : Char = ' ') : string;
type pword   = ^word;
var  i, ideb : integer;
     ch      : string;
     pval    : word;
begin
  ch     := StringOfChar(Car, Corpus);
  result := Text;
  ideb   := Length(Text) - 1;
  for i := ideb downto 1 do
    begin
      pVal := (pword(@result[i]))^;
      if (pVal = $0a0d) then Insert(ch, result, i + 2);
    end;
  result := StringOfChar(Car, FirstLine) + result;
end;

//-------------------------------------------------------------------------------------------------
// CaseString
//-------------------------------------------------------------------------------------------------
// PURPOSE  : Search a string in a strings array (case insensitive)
// IN       : Text -> String to search
//            List -> Strings array
// OUT      : Index in the array, starting from 0 (-1 if not found)
//-------------------------------------------------------------------------------------------------
function CaseString(Text : string; List : array of string) : integer;
var i : integer;
begin
  result := -1;
  for i := Low(List) to High(List) do
    if SameText(List[i], Text) then
      begin
        result := i;
        break;
      end;
end;

//-------------------------------------------------------------------------------------------------
// CaseObject
//-------------------------------------------------------------------------------------------------
// PURPOSE  : Search an object pointeur value in an object pointeurs array.
// IN       : Obj  -> Pointer to search
//            List -> Pointers array
// OUT      : Index  partir de 0 / -1 si pas trouv
//-------------------------------------------------------------------------------------------------
function CaseObject(Obj : TObject; List : array of TObject) : integer;
var i : integer;
begin
  result := -1;
  for i := Low(List) to High(List) do
    if (List[i] = Obj) then
      begin
        result := i;
        break;
      end;
end;

//-------------------------------------------------------------------------------------------------
// FreeObject
//-------------------------------------------------------------------------------------------------
// PURPOSE  : Secure object freeing and set it to nil.
// EXCEPT   : EAbort (If error, message is displayed and Abort raised)
//-------------------------------------------------------------------------------------------------
procedure FreeObject(var Obj);
var p : TObject;
    s : string;
begin
  try
    s := '';
    p := TObject(Obj);
    if (p = nil) then Exit;
    try
      if (p is TComponent) then
        with (p as TComponent) do
          begin
            if (csDestroying in ComponentState) then Exit;
            s := Name + ': ';
          end;
      s := s + p.ClassName;
    except
    end;
    TObject(Obj) := nil;
    p.Free;
  except
    on E : Exception do
      begin
        if (s = '') then s := XAM_CAPTION_UNKNOWN;
        MessageAbort(Format('Error on destroying objet [%s] :' + ENDL2 + '%s', [s, E.Message]));
      end;
  end;
end;

//-------------------------------------------------------------------------------------------------
// VarToDisp
//-------------------------------------------------------------------------------------------------
// PURPOSE  : Convert a variant to an IDispatch pointer.
//-------------------------------------------------------------------------------------------------
function VarToDisp(Value : Variant) : IDispatch;
begin
  result := IDispatch(TVarData(Value).VDispatch);
end;

//-------------------------------------------------------------------------------------------------
// VarToText
//-------------------------------------------------------------------------------------------------
// PURPOSE  : Convert a Variant to a string.
//-------------------------------------------------------------------------------------------------
function VarToText(Value : variant) : string;
const Tab : array[boolean] of string = ( 'False', 'True' );
begin
  case VarIsNull(Value) of
    True  : result := 'NULL';
    False : if VarIsArray(Value) then result := 'ARRAY';
  end;
  case VarType(Value) of
    varEmpty   : result := 'UNASSIGNED';
    varNull    : result := 'NULL';
    varBoolean : result := Tab[boolean(Value)];
  else
    result := VarToStr(Value);
  end;
end;

//-------------------------------------------------------------------------------------------------
// VarRecToText
//-------------------------------------------------------------------------------------------------
// PURPOSE  : Convert a TVarRec to a string.
//-------------------------------------------------------------------------------------------------
function VarRecToText(VarRec : TVarRec) : string;
begin
  case VarRec.VType of
    vtInteger   : result := VarToStr(VarRec.VInteger);
    vtBoolean   : result := VarToStr(VarRec.VBoolean);
    vtChar      : result := QuotedStr(VarRec.VChar);
    vtExtended  : result := VarToStr(VarRec.VExtended^);
    vtString    : result := QuotedStr(VarRec.VString^);
    vtPointer   : result := '0x' + IntToHex(integer(VarRec.VPointer), 16);
    vtPChar     : result := QuotedStr(VarToStr(VarRec.VPChar^));
    vtObject    : result := TXAMObjectInfos.GetObjectInfos(VarRec.VObject);
    vtClass     : result := 'CLASS of ' + VarRec.VClass.ClassName;
    vtWideChar  : result := QuotedStr(VarRec.VWideChar);
    vtPWideChar : result := QuotedStr(VarRec.VPWideChar^);
    vtAnsiString: result := QuotedStr(string(VarRec.VAnsiString));
    vtCurrency  : result := VarToStr(VarRec.VCurrency^);
    vtVariant   : result := 'VARIANT';
    vtInterface : result := 'INTERFACE';
    vtWideString: result := QuotedStr(string(VarRec.VWideString));
    {$ifdef VER140}
    vtInt64     : result := VarToStr(VarRec.VInt64^);
    {$endif}
    {$ifdef VER150}
    vtInt64     : result := VarToStr(VarRec.VInt64^);
    {$endif}
  end;
end;

{
<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<->>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
                                                        Registry
<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<->>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
}

//-------------------------------------------------------------------------------------------------
// RegOpenKey
//-------------------------------------------------------------------------------------------------
function RegOpenKey(Reg : TRegistry; Key : String; CanCreate : boolean = true; DoRaise : boolean = true) : boolean;
begin
  if not Assigned(Reg) then Abort;
  Reg.CloseKey;
  result := Reg.OpenKey(Key, CanCreate);
  if (not result) and (Doraise) then MessageAbort(Format(XAMERR_ACCESS_REG, [Key]));
end;

//-------------------------------------------------------------------------------------------------
// RegRead...
//-------------------------------------------------------------------------------------------------
function RegReadInteger(Reg : TRegistry; Name : String; Default : integer) : integer;
begin
  if not Assigned(Reg) then Abort;
  case Reg.ValueExists(Name) of
    True  : result := Reg.ReadInteger(Name);
    False : begin
              Reg.WriteInteger(Name, Default);
              result := Default;
            end;
  end;
end;

function RegReadBool(Reg : TRegistry; Name : String; Default : boolean) : boolean;
begin
  if not Assigned(Reg) then Abort;
  case Reg.ValueExists(Name) of
    True  : result := Reg.ReadBool(Name);
    False : begin
              Reg.WriteBool(Name, Default);
              result := Default;
            end;
  end;
end;

function RegReadString(Reg : TRegistry; Name : String; Default : string) : string;
begin
  if not Assigned(Reg) then Abort;
  case Reg.ValueExists(Name) of
    True  : result := Reg.ReadString(Name);
    False : begin
              Reg.WriteString(Name, Default);
              result := Default;
            end;
  end;
end;

{
<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<->>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
                                                       SystemFile
<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<->>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
}

//-------------------------------------------------------------------------------------------------
// CreateEmptyFile
//-------------------------------------------------------------------------------------------------
// PURPOSE  : Create an empty file.
//-------------------------------------------------------------------------------------------------
procedure CreateEmptyFile(s : string);
begin
  with TStringList.Create do
    try
      SaveToFile(s);
    finally
      Free;
    end;
end;

//-------------------------------------------------------------------------------------------------
// CheckFileExists
//-------------------------------------------------------------------------------------------------
procedure CheckFileExists(FileName : string);
begin
  if not FileExists(FileName) then MessageAbort(Format(XAMERR_FILE_NOT_FOUND, [FileName]));
end;

//-------------------------------------------------------------------------------------------------
// IsFileInUse
//-------------------------------------------------------------------------------------------------
function IsFileInUse(FileName : string ) : boolean;
var HFileRes : HFILE;
begin
   result := false;
   if not FileExists(FileName) then Exit;
   HFileRes := CreateFile(pchar(FileName), GENERIC_READ or GENERIC_WRITE, 0 {"the trick"}, nil, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);
   result   := (HFileRes = INVALID_HANDLE_VALUE);
   if (not result) then CloseHandle(HFileRes);
end;

//-------------------------------------------------------------------------------------------------
// ExecuteFile
//-------------------------------------------------------------------------------------------------
// PURPOSE  : Execute a processus.
// IN       : FileName   -> Full path name
//            Params     -> Command line parameters
//            DefaultDir -> Default path
//            ShowCmd    -> Launch mode
// OUT      : Return the processus handle
// EXCEPT   : EAbort
//-------------------------------------------------------------------------------------------------
function ExecuteFile(FileName : string; Shell : boolean = false; Params : string = ''; DefaultDir : string = ''; ShowCmd : word = SW_SHOW) : THandle;
const tab : array[boolean] of string = ( '', 'open' );
var s : string;
begin
  result := ShellExecute(Application.Handle, PChar(tab[shell]), PChar(FileName), PChar(Params), PChar(DefaultDir), ShowCmd);
  if (result > 32) then Exit;
  case (result) of
    0	                      : s := 'Not enough memory or system ressources';
    SE_ERR_DLLNOTFOUND,
    SE_ERR_FNF              : s := 'File not found';
    SE_ERR_PNF              : s := 'Path not found';
    ERROR_BAD_FORMAT	      : s := 'Invalid file';
    SE_ERR_ACCESSDENIED	    : s := 'Access denied';
    SE_ERR_ASSOCINCOMPLETE	: s := 'Bad file association';
    SE_ERR_NOASSOC	        : s := 'No association found';
    SE_ERR_OOM	            : s := 'Not enough memory';
    SE_ERR_SHARE	          : s := 'Share violation';
  else
    s := IntToStr(result);
  end;
  MessageAbort('Error on executing :' + ENDL2 +
               '   File : ' + ExtractFileName(FileName) + ENDL +
               '   Params : ' + Params + ENDL +
               '   Path : ' + ExtractFilePath(FileName) + ENDL2 +
               '   Message : ' + s);
end;

//-------------------------------------------------------------------------------------------------
// WinExec32
//-------------------------------------------------------------------------------------------------
// PURPOSE  : Execute a processus and wait for its termination.
// IN       : FileName -> Full path name
//            ShowCmd  -> Launch mode
//            Wait     -> True for waiting termination, false if not
// OUT      : Return the processus exit code, 0 if launch faild or nowait.
//-------------------------------------------------------------------------------------------------
function WinExec32(FileName : string; ShowCmd : word; Wait : boolean) : dword;
var zAppName    : array[0..512] of char;
    StartupInfo : TStartupInfo;
    ProcessInfo : TProcessInformation;
begin
  result := 0;
  StrPCopy(zAppName, FileName);
  FillChar(StartupInfo, Sizeof(StartupInfo), #0);
  StartupInfo.cb          := Sizeof(StartupInfo);
  StartupInfo.dwFlags     := STARTF_USESHOWWINDOW;
  StartupInfo.wShowWindow := ShowCmd;
  case (CreateProcess(nil, zAppName, nil, nil, false, CREATE_NEW_CONSOLE, nil, nil, StartupInfo, ProcessInfo) <> False) of
    True  : begin
              if (Wait) then
                begin
                  WaitForSingleObject(ProcessInfo.hProcess, INFINITE);
                  GetExitCodeProcess(ProcessInfo.hProcess, result);
                end;
              CloseHandle(ProcessInfo.hProcess);
              CloseHandle(ProcessInfo.hThread);
            end;
    False :
  end;
end;

//-------------------------------------------------------------------------------------------------
// GetFileSize
//-------------------------------------------------------------------------------------------------
// PURPOSE  : Get file size.
// IN       : FileName -> Path
// OUT      : return the size in bytes
// EXCEPT   : EBSToolsFileNotFound
//-------------------------------------------------------------------------------------------------
function GetFileSize(FileName : string) : integer;
var rec : TSearchRec;
begin
  if (FindFirst(FileName, faAnyFile, rec) <> 0) then
    MessageAbort('File not found : ' + ENDL +
                 '   Source : ' + FileName);
  result := rec.Size;
  SysUtils.FindClose(rec);
end;

//-------------------------------------------------------------------------------------------------
// DoCopyFile
//-------------------------------------------------------------------------------------------------
// PURPOSE  : Safe file copy.
// IN       : Src -> Source
//            Dest -> Destination
// EXCEPT   : EAbort
//-------------------------------------------------------------------------------------------------
procedure DoCopyFile(Src, Dest : string);
begin
  CheckFileExists(Src);
  try
    Win32Check( CopyFile(PChar(Src), PChar(Dest), true) );
  except
    on E : Exception do MessageAbort('Error on copy file : ' + ENDL +
                                     '   Source : ' + Src + ENDL +
                                     '   Destination : ' + Dest + ENDL2 +
                                     E.Message);
  end;
end;

//-------------------------------------------------------------------------------------------------
// DoMoveFile
//-------------------------------------------------------------------------------------------------
// PURPOSE  : Save file move.
// IN       : Src -> Source
//            Dest -> Destination
// EXCEPT   : EAbort
//-------------------------------------------------------------------------------------------------
procedure DoMoveFile(Src, Dest : string);
begin
  CheckFileExists(Src);
  try
    Win32Check( MoveFile(PChar(Src), PChar(Dest)) );
  except
    on E : Exception do MessageAbort('Error on move file : ' + ENDL +
                                     '   Source : ' + Src + ENDL +
                                     '   Destination : ' + Dest + ENDL2 +
                                     E.Message);
  end;
end;

//-------------------------------------------------------------------------------------------------
// DoDeleteFile
//-------------------------------------------------------------------------------------------------
// PURPOSE  : Safe file delete.
// IN       : Src -> Fichier
// EXCEPT   : EAbort
//-------------------------------------------------------------------------------------------------
procedure DoDeleteFile(Src : string);
begin
  if not FileExists(Src) then Exit;
  try
    Win32Check( DeleteFile(PChar(Src)) );
  except
    on E : Exception do MessageAbort('Error on delete file : ' + ENDL +
                                     '   Source : ' + Src + ENDL2 +
                                     E.Message);
  end;
end;

//-------------------------------------------------------------------------------------------------
// GetLongFileName
//-------------------------------------------------------------------------------------------------
// PURPOSE  : Convert a 8.3 filename into LFN.
//-------------------------------------------------------------------------------------------------
function GetLongFileName(FileName : string) : string;
var i    : integer;
    r    : TSearchRec;
    s, p : string;
begin
  with TStringList.Create do
    try
      Add( Copy(FileName, 1, 2) );
      System.Delete(FileName, 1, 3);
      repeat
        i := Pos('\', FileName);
        case (i = 0) of
          True  : Add(FileName);
          False : begin
                    Add( Copy(FileName, 1, i - 1) );
                    System.Delete(FileName, 1, i);
                  end;
        end;
      until (i = 0);
      s := Strings[0] + '\';
      p := s;
      for i := 1 to Pred(Count) do
        begin
          s := s + Strings[i];
          FindFirst(s, faAnyFile, r);
          p := p + IncludeTrailingBackslash(r.Name);
          s := p;
        end;
      FindClose(r);
      result := ExcludeTrailingBackslash(s);
    finally
      Free;
    end;
end;

{
<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<->>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
                                                    TXAMObjectInfos
<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<->>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
}

//-------------------------------------------------------------------------------------------------
// TXAMObjectInfos.GetObjectInfos
//-------------------------------------------------------------------------------------------------
// PURPOSE  : return a string that describe an object.
//-------------------------------------------------------------------------------------------------
class function TXAMObjectInfos.GetObjectInfos(AObject : TObject) : string;
begin
  with (Create) do
    try
      Target := AObject;
      result := Text;
    finally
      Free;
    end;
end;

//-------------------------------------------------------------------------------------------------
// TXAMObjectInfos.SetTarget
//-------------------------------------------------------------------------------------------------
procedure TXAMObjectInfos.SetTarget(Value : TObject);
var p : TClass;
    o : TComponent;
begin
  FComponentName   := XAM_CAPTION_NA;
  FTargetClassName := XAM_CAPTION_NA;
  FParentClassName := XAM_CAPTION_NA;
  FInherits        := XAM_CAPTION_NA;
  FOwners          := XAM_CAPTION_NA;
  FText            := XAM_CAPTION_NA;
  FTarget          := Value;
  try
    try
      if (FTarget <> nil) and Assigned(FTarget) then
        try
          FTargetClassName := FTarget.ClassName;
          if (FTarget is TComponent) then
            with (FTarget as TComponent) do
              begin
                case (Name = '') of
                  True  : FComponentName := XAM_CAPTION_UNKNOWN;
                  False : FComponentName := Name;
                end;
              if (Owner <> nil) then
                begin
                  FOwners := Format('%s: %s ( %s )', [Owner.Name, Owner.ClassName, Owner.ClassParent.ClassName]);
                  o       := Owner;
                  while (o.Owner <> nil) do
                    begin
                      FOwners := Format('%s -> %s: %s ( %s )', [FOwners, o.Owner.Name, o.Owner.ClassName, o.Owner.ClassParent.ClassName]);
                      o       := o.Owner;
                    end;
                end;
            end;
          if (FTarget.ClassParent <> nil) then
            begin
              FParentClassName := FTarget.ClassParent.ClassName;
              FInherits        := FTargetClassName + ' > ' + FParentClassName;
              p                := FTarget.ClassParent;
              while (p.ClassParent <> nil) do
                begin
                  FInherits := FInherits + ' > ' + p.ClassParent.ClassName;
                  p         := p.ClassParent;
                end;
            end;
        except
        end;
    finally
      FText := Format('%s: %s ( %s )', [FComponentName, FTargetClassName, FParentClassName]);
    end;
  except
  end;
end;

end.

