{
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
PLATFORM  : Windows 9x/NT/XP - Delphi 5,6,7
PROJECT   : eXpress Application Manager
FILE      : U_XAM_System.pas
PURPOSE   : Windows & Application classes.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
LICENSE   : LGPL
AUTHOR    : Copyright 2003 Olivier Rogier 
WEBSITE   : http://www.ordisoftware.info
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CREATION  : v 1.0.0 - 01/2003
REVISION  : v 1.0.1 - 02/2003
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
}

unit U_XAM_System;

{
#######################################################################################################################
                                                       INTERFACE
#######################################################################################################################
}

interface

uses
  Registry, Windows, Classes, Messages, U_XAM_LogFile;

type
  TXAMAppState         = ( asNone, asInit, asRun, asEnd );                                // Application state
  TXAMWinPlatform      = ( win9x, winNT );                                                // Platform type
  TXAMWinExitMode      = ( ewLogOff, ewReboot, ewShutDown, ewPowerOff, ewForce );         // Windows shutdown mode
  TXAMAppPriority      = ( pcHigh, pcIdle, pcNormal, pcRealtime );                        // Thread priority
  TXAMEndSessionNotify = procedure (Sender : TObject; var AllowExit : boolean) of object; // Windows end session event

  // **********************************************************************************************
  // TXAMWindows
  // **********************************************************************************************
  // PURPOSE : Windows system informations.
  // **********************************************************************************************
  TXAMWindows = class
  private
    FPlatform             : TXAMWinPlatform;    
    FComputer             : string;             
    FUser                 : string;             
    FIP                   : string;
    FVersion              : string;             
    FEnvVar               : TStrings;           
    FVideoModes           : TStrings;           
    FPathWin              : string;             
    FPathSys              : string;             
    FPathTemp             : string;             
    FPathProgramFiles     : string;             
    FPathCommonFiles      : string;
    FPathMedia            : string;
    FPathCommonAdminTools : string;
    FPathCommonAppData    : string;
    FPathCommonDesktop    : string;
    FPathCommonDocuments  : string;
    FPathCommonFavorites  : string;
    FPathCommonPrograms   : string;
    FPathCommonStartMenu  : string;
    FPathCommonStartup    : string;
    FPathCommonTemplates  : string;
    FPathAdminTools       : string;
    FPathAppData          : string;
    FPathCookies          : string;
    FPathDesktop          : string;
    FPathDocuments        : string;
    FPathFavorites        : string;
    FPathFonts            : string;
    FPathHistory          : string;
    FPathLocalSettings    : string;
    FPathLocalAppData     : string;
    FPathNetCache         : string;
    FPathNetHood          : string;
    FPathPictures         : string;
    FPathPrintHood        : string;
    FPathPrograms         : string;
    FPathRecent           : string;
    FPathSendTo           : string;
    FPathStartMenu        : string;
    FPathStartup          : string;
    FPathTemplates        : string;
    FMemUsed              : string;
    FMemPhysTotal         : string;
    FMemPhysAvail         : string;
    FMemPageFileTotal     : string;
    FMemPageFileAvail     : string;
    FMemVirtualTotal      : string;
    FMemVirtualAvail      : string;
    FOnEndSession         : TXAMEndSessionNotify;
  protected
    function        HookProc(var Message : TMessage) : boolean; virtual;
  public
    constructor     Create; virtual;
    destructor      Destroy; override;
    procedure       Refresh; virtual;
    procedure       RefreshMemoryStatus;
    class procedure DoExit(Action : TXAMWinExitMode);
    class procedure SetTaskBar(Visible, Enabled : boolean);
    class procedure GetDLLFunctions(FileName : string; List : TStrings);
    class procedure SetAppRun(Description, FileName : string; State, RunOnce : boolean);
    class procedure SetScreenSaverState(Active : boolean);
    class procedure SetSystemKeys(State : boolean);
    class procedure SetSystemAltTabKeys(State : boolean);
  public
    property Platform             : TXAMWinPlatform       read FPlatform;
    property Computer             : string                read FComputer;
    property User                 : string                read FUser;
    property IP                   : string                read FIP;
    property Version              : string                read FVersion;
    property EnvVar               : TStrings              read FEnvVar;
    property VideoModes           : TStrings              read FVideoModes;
    property PathWin              : string                read FPathWin;
    property PathSys              : string                read FPathSys;
    property PathTemp             : string                read FPathTemp;
    property PathProgramFiles     : string                read FPathProgramFiles;
    property PathCommonFiles      : string                read FPathCommonFiles;
    property PathMedia            : string                read FPathMedia;
    property PathCommonAdminTools : string                read FPathCommonAdminTools;
    property PathCommonAppData    : string                read FPathCommonAppData;
    property PathCommonDesktop    : string                read FPathCommonDesktop;
    property PathCommonDocuments  : string                read FPathCommonDocuments;
    property PathCommonFavorites  : string                read FPathCommonFavorites;
    property PathCommonPrograms   : string                read FPathCommonPrograms;
    property PathCommonStartMenu  : string                read FPathCommonStartMenu;
    property PathCommonStartup    : string                read FPathCommonStartup;
    property PathCommonTemplates  : string                read FPathCommonTemplates;
    property PathAdminTools       : string                read FPathAdminTools;
    property PathAppData          : string                read FPathAppData;
    property PathCookies          : string                read FPathCookies;
    property PathDesktop          : string                read FPathDesktop;
    property PathDocuments        : string                read FPathDocuments;
    property PathFavorites        : string                read FPathFavorites;
    property PathFonts            : string                read FPathFonts;
    property PathHistory          : string                read FPathHistory;
    property PathLocalSettings    : string                read FPathLocalSettings;
    property PathLocalAppData     : string                read FPathLocalAppData;
    property PathNetCache         : string                read FPathNetCache;
    property PathNetHood          : string                read FPathNetHood;
    property PathPictures         : string                read FPathPictures;
    property PathPrintHood        : string                read FPathPrintHood;
    property PathPrograms         : string                read FPathPrograms;
    property PathRecent           : string                read FPathRecent;
    property PathSendTo           : string                read FPathSendTo;
    property PathStartMenu        : string                read FPathStartMenu;
    property PathStartup          : string                read FPathStartup;
    property PathTemplates        : string                read FPathTemplates;
    property MemUsed              : string                read FMemUsed;
    property MemPhysTotal         : string                read FMemPhysTotal;
    property MemPhysAvail         : string                read FMemPhysAvail;
    property MemPageFileTotal     : string                read FMemPageFileTotal;
    property MemPageFileAvail     : string                read FMemPageFileAvail;
    property MemVirtualTotal      : string                read FMemVirtualTotal;
    property MemVirtualAvail      : string                read FMemVirtualAvail;
    property OnEndSession         : TXAMEndSessionNotify  read FOnEndSession      write FOnEndSession;
  end;

  // **********************************************************************************************
  // TXAMApplication
  // **********************************************************************************************
  // PURPOSE : Application process informations.
  // **********************************************************************************************
  TXAMApplication = class
  private
    FDelphiRunning  : boolean;          // True if Delphi processus is running
    FDelphiOwner    : boolean;          // True if Delphi is the current application process (like package)
    FRunByDebugger  : boolean;          // true if the application is run by a debugger
    FCmdLine        : TStringList;      // Command line parameters list
    FTitle          : string;           // Application title
    FTitleWithVer   : string;           // Application title with version
    FVersion        : string;           // version string
    FFileName       : string;           // Application path in LFN
    FExt            : string;           // Application code
    FPathBin        : string;           // Application executable path
    FPathRoot       : string;           // Application executable parent folder
    FState          : TXAMAppState;     // Application status
    FLogFile        : TXAMLogFile;      // Logfile
    FMutex          : THandle;          // Application mutex (for check only one instance)
  private
    procedure   SetPriority(Value : TXAMAppPriority); virtual;
    function    GetPriority : TXAMAppPriority; virtual;
  public
    constructor Create;
    destructor  Destroy; override;
    procedure   ShowMessageLog(Msg : string);
    function    TestOnlyOneInstance : boolean;
  public
    property DelphiRunning  : boolean          read FDelphiRunning;
    property DelphiOwner    : boolean          read FDelphiOwner;
    property RunByDebugger  : boolean          read FRunByDebugger;
    property CmdLine        : TStringList      read FCmdLine;
    property Title          : string           read FTitle;
    property TitleWithVer   : string           read FTitleWithVer;
    property Version        : string           read FVersion;
    property FileName       : string           read FFileName;
    property Ext            : string           read FExt;
    property PathBin        : string           read FPathBin;
    property PathRoot       : string           read FPathRoot;
    property Priority       : TXAMAppPriority  read GetPriority     write SetPriority;
    property State          : TXAMAppState     read FState          write FState;
    property LogFile        : TXAMLogFile      read FLogFile;
  end;

var
  XAMWindows       : TXAMWindows     = nil;
  XAMApplication   : TXAMApplication = nil;

procedure XAMInitializeSystem;

{
#######################################################################################################################
                                                     IMPLEMENTATION
#######################################################################################################################
}

implementation

uses
  MMSystem, Dialogs, SysUtils, Forms, SysConst, WinSock, ImageHlp, U_XAM_Global, U_XAM_Debug, U_XAM_FileVersion;

const
  TAB_NAMES_PLATFORM    : array[0..2] of string = ( 'Win32s on Windows 3.1', 'Win32 on Windows 95', 'Windows NT' );
  TAB_PLATFORM          : array[boolean] of TXAMWinPlatform = ( win9x, winNT );

  TAB_ROOTKEY_VALUES    : array[0..6] of dword
                        = ( HKEY_CLASSES_ROOT, HKEY_CURRENT_USER, HKEY_LOCAL_MACHINE, HKEY_USERS,
                            HKEY_PERFORMANCE_DATA, HKEY_CURRENT_CONFIG, HKEY_DYN_DATA );
  TAB_ROOTKEY_NAMES     : array[0..6] of string
                        = ( 'HKEY_CLASSES_ROOT', 'HKEY_CURRENT_USER', 'HKEY_LOCAL_MACHINE', 'HKEY_USERS',
                            'HKEY_PERFORMANCE_DATA', 'HKEY_CURRENT_CONFIG', 'HKEY_DYN_DATA' );

  REGKEY_WINDOWS_CURRENT_VERSION = 'Software\Microsoft\Windows\CurrentVersion';
  REGKEY_WINDOWS_RUN             = REGKEY_WINDOWS_CURRENT_VERSION + '\Run';
  REGKEY_WINDOWS_SHELL_FOLDERS   = REGKEY_WINDOWS_CURRENT_VERSION + '\Explorer\Shell Folders';

  TAB_PRIORITY_CLASS_LIST : array[TXAMAppPriority] of dword
                          = ( HIGH_PRIORITY_CLASS, IDLE_PRIORITY_CLASS, NORMAL_PRIORITY_CLASS, REALTIME_PRIORITY_CLASS );

//-------------------------------------------------------------------------------------------------
// XAMInitializeSystem
//-------------------------------------------------------------------------------------------------
procedure XAMInitializeSystem;
begin
  try
    if not Assigned(XAMWindows)     then XAMWindows     := TXAMWindows.Create;
    if not Assigned(XAMApplication) then XAMApplication := TXAMApplication.Create;
    if Assigned(XAMApplication) and Assigned(XAMApplication.LogFile) then XAMApplication.LogFile.Enabled := XAM_LOGFILE;
  except
    on E : Exception do
      case (ModuleIsPackage) of
        True  : ShowMessage('XAMApplication initialisation error : ' + ENDL2 + E.Message);
        False : raise;
      end;
  end;
end;

{
<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<->>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
                                                      TXAMWindows
<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<->>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
}

//-------------------------------------------------------------------------------------------------
// TXAMWindows.Create
//-------------------------------------------------------------------------------------------------
constructor TXAMWindows.Create;
begin
  FEnvVar     := TStringList.Create;
  FVideoModes := TStringList.Create;
  Application.HookMainWindow(HookProc);
  try
    Refresh;
  except
    if (not ModuleIsPackage) then raise;
  end;
end;

//-------------------------------------------------------------------------------------------------
// TXAMWindows.Create
//-------------------------------------------------------------------------------------------------
destructor TXAMWindows.Destroy;
begin
  Application.UnhookMainWindow(HookProc);
  FreeObject(FEnvVar);
  FreeObject(FVideoModes);
  inherited;
end;

//-------------------------------------------------------------------------------------------------
// TXAMWindows.Refresh
//-------------------------------------------------------------------------------------------------
procedure TXAMWindows.Refresh;
var z  : array [0..MAX_PATH] of char;
    ln : dword;
    ///////////////////////////////////////////////////////////////////////////
    procedure Win32GetValue(RetVal : boolean; var s : string);
    var i : dword;
    begin
      case (integer(RetVal) > 0) of
        True  : s := z;
        False : begin
                  i := GetLastError;
                  case (i <> ERROR_SUCCESS) of
                    True  : s := SysErrorMessage(i);
                    False : s := 'A call to an OS function failed';
                  end;
                end;
      end;
    end;
    ///////////////////////////////////////////////////////////////////////////
type TaPInAddr = array [0..10] of PInAddr;
     PaPInAddr = ^TaPInAddr;
var p, q      : PChar;
    dm        : TDeviceMode;
    i         : integer;
    GInitData : TWSADATA;
    phe       : PHostEnt;
    pptr      : PaPInAddr;
    s         : string;
begin
  // Get Windows Version
  FPlatform := TAB_PLATFORM[Win32Platform = 2];
  FVersion  := Format('%s v%d.%d.%d %s', [TAB_NAMES_PLATFORM[Win32Platform], Win32MajorVersion,
                                          Win32MinorVersion, Win32BuildNumber, Win32CSDVersion]);
  // Get Login Informations
  ln := MAX_PATH; Win32GetValue(GetComputerName(z, ln), FComputer);
  ln := MAX_PATH; Win32GetValue(GetUserName(z, ln),     FUser);
  // Get Folder Informations
  ln := MAX_PATH;
  Win32GetValue( boolean( GetWindowsDirectory(z, ln) ), FPathWin);
  Win32GetValue( boolean( GetSystemDirectory(z, ln) ),  FPathSys );
  Win32GetValue( boolean( GetTempPath(ln, z) ),         FPathTemp );
  with (TRegistry.Create) do
    try
      RootKey := HKEY_LOCAL_MACHINE;
      OpenKey(REGKEY_WINDOWS_CURRENT_VERSION, false);
      FPathProgramFiles     := ReadString('ProgramFilesDir');
      FPathCommonFiles      := ReadString('CommonFilesDir');
      FPathMedia            := ReadString('MediaPath');
      OpenKey(REGKEY_WINDOWS_SHELL_FOLDERS, false);
      FPathCommonAdminTools := ReadString('Common Administrative Tools');
      FPathCommonAppData    := ReadString('Common AppData');
      FPathCommonDesktop    := ReadString('Common Desktop');
      FPathCommonDocuments  := ReadString('Common Documents');
      FPathCommonFavorites  := ReadString('Common Favorites');
      FPathCommonPrograms   := ReadString('Common Programs');
      FPathCommonStartMenu  := ReadString('Common Start Menu');
      FPathCommonStartup    := ReadString('Common Startup');
      FPathCommonTemplates  := ReadString('Common Templates');
      RootKey := HKEY_CURRENT_USER;
      OpenKey(REGKEY_WINDOWS_SHELL_FOLDERS, false);
      FPathAdminTools       := ReadString('Administrative Tools');
      FPathAppData          := ReadString('AppData');
      FPathCookies          := ReadString('Cookies');
      FPathDesktop          := ReadString('Desktop');
      FPathDocuments        := ReadString('Personal');
      FPathFavorites        := ReadString('Favorites');
      FPathFonts            := ReadString('Fonts');
      FPathHistory          := ReadString('History');
      FPathLocalSettings    := ReadString('Local Settings');
      FPathLocalAppData     := ReadString('Local AppData');
      FPathNetCache         := ReadString('Cache');
      FPathNetHood          := ReadString('NetHood');
      FPathPictures         := ReadString('My Pictures');
      FPathPrintHood        := ReadString('PrintHood');
      FPathPrograms         := ReadString('Programs');
      FPathRecent           := ReadString('Recent');
      FPathSendTo           := ReadString('SendTo');
      FPathStartMenu        := ReadString('Start Menu');
      FPathStartup          := ReadString('Startup');
      FPathTemplates        := ReadString('Templates');
    finally
      Free;
    end;
  // Environment Variables Informations
  p := GetEnvironmentStrings;
  q := p;
  FEnvVar.Clear;
  repeat
    FEnvVar.Add(StrPas(p));
    inc(p, StrLen(p) + 1);
  until (p^ = #0);
  FreeEnvironmentStrings(q);
  // Video Mode Informations
  FVideoModes.Clear;
  i := 0;
  while EnumDisplaySettings(nil, i, dm) do
    begin
      s := Format('%d x %d (%d bits)', [dm.dmPelsWidth, dm.dmPelsHeight, dm.dmBitsPerPel]);
      if (FVideoModes.IndexOf(s) = -1) then FVideoModes.Add(s);
      Inc(i);
    end;
  // IP Address
  FIP := XAM_CAPTION_UNKNOWN;
  WSAStartup($101, GInitData);
  GetHostName(z, SizeOf(z));
  phe := GetHostByName(z);
  if (phe = nil) then Exit;
  pptr := PaPInAddr(Phe^.h_addr_list);
  i := 0;
  while (pptr^[i] <> nil) do
    begin
      FIP := StrPas(inet_ntoa(pptr^[i]^));
      Inc(i);
    end;
  WSACleanup;
  // Memory Informations
  RefreshMemoryStatus;
end;

//-------------------------------------------------------------------------------------------------
// TXAMWindows.RefreshMemStatus
//-------------------------------------------------------------------------------------------------
procedure TXAMWindows.RefreshMemoryStatus;
var ms : TMemoryStatus;
begin
  ms.dwLength := sizeof(ms);
  GlobalMemoryStatus(ms);
  FMemUsed          := Format('%d',      [ms.dwMemoryLoad]) + ' %';
  FMemPhysTotal     := Format('%.0n Ko', [ms.dwTotalPhys     / 1024]);
  FMemPhysAvail     := Format('%.0n Ko', [ms.dwAvailPhys     / 1024]);
  FMemPageFileTotal := Format('%.0n Ko', [ms.dwTotalPageFile / 1024]);
  FMemPageFileAvail := Format('%.0n Ko', [ms.dwAvailPageFile / 1024]);
  FMemVirtualTotal  := Format('%.0n Ko', [ms.dwTotalVirtual  / 1024]);
  FMemVirtualAvail  := Format('%.0n Ko', [ms.dwAvailVirtual  / 1024]);
end;

//-------------------------------------------------------------------------------------------------
// TXAMWindows.HookProc
//-------------------------------------------------------------------------------------------------
function TXAMWindows.HookProc(var Message : TMessage) : boolean;
var b : boolean;
begin
  Result := false;
  case (Message.Msg) of
    WM_ENDSESSION     : if TWMEndSession(Message).EndSession then Halt;
    WM_QUERYENDSESSION: begin
                          b := WordBool(Message.Result);
                          if Assigned(FOnEndSession) then
                            begin
                              FOnEndSession(self, b);
                              Message.Result := integer(b);
                            end;
                        end;
  end;
end;

//-------------------------------------------------------------------------------------------------
// TXAMWindows.DoExit
//-------------------------------------------------------------------------------------------------
class procedure TXAMWindows.DoExit(Action : TXAMWinExitMode);
begin
  case (Action) of
    ewLogOff   : Win32Check(ExitWindowsEx(EWX_LOGOFF,   0));
    ewReboot   : Win32Check(ExitWindowsEx(EWX_REBOOT,   0));
    ewShutDown : Win32Check(ExitWindowsEx(EWX_SHUTDOWN, 0));
    ewPowerOff : Win32Check(ExitWindowsEx(EWX_POWEROFF, 0));
    ewForce    : Win32Check(ExitWindowsEx(EWX_FORCE,    0));
  end;
end;

//-------------------------------------------------------------------------------------------------
// TXAMWindows.SetTaskBar
//-------------------------------------------------------------------------------------------------
class procedure TXAMWindows.SetTaskBar(Visible, Enabled : boolean);
const vTab : array[boolean] of integer = ( SW_HIDE, SW_RESTORE );
var h : THandle;
begin
  h := FindWindow('Shell_TrayWnd', nil);
  EnableWindow(h, Enabled);
  ShowWindow(h, vTab[Visible]);
end;

//-------------------------------------------------------------------------------------------------
// TXAMWindows.GetDLLFunctions
//-------------------------------------------------------------------------------------------------
class procedure TXAMWindows.GetDLLFunctions(FileName : string; List : TStrings);
type SegPtr = array[0..$FFFFFF] of char;
var h       : THandle;
    i, fc   : integer;
    s       : string;
    arr     : pointer;
    idbginf : PImageDebugInformation;
begin
  List.Clear;
  FileName := ExpandFileName(FileName);
  if not FileExists(FileName) then raise Exception.CreateFmt('File not found : %s', [FileName]);
  h := CreateFile(PChar(FileName), GENERIC_READ, FILE_SHARE_READ or FILE_SHARE_WRITE, nil, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);
  if (h = INVALID_HANDLE_VALUE) then raise Exception.CreateFmt('Invalid file : %s', [FileName]);
  try
    idbginf := MapDebugInformation(h, PChar(FileName), nil,0);
    if (idbginf = nil) then raise Exception.CreateFmt('Invalid file : %s', [FileName]);
    try
      arr := (idbginf^).ExportedNames;
      fc  := 0;
      for i := 0 to Pred((idbginf^).ExportedNamesSize) do
        if SegPtr(arr^)[i] = #0 then
          begin
            s := PChar(@SegPtr(arr^)[fc]);
            if (Length(s) > 0) then List.Add(s);
            if (i > 0) and (SegPtr(arr^)[i - 1] = #0) then Break;
            fc := i + 1;
          end;
    finally
      UnmapDebugInformation(idbginf);
    end;
  finally
    CloseHandle(h);
  end;
end;

//-------------------------------------------------------------------------------------------------
// TXAMWindows.SetAppRun
//-------------------------------------------------------------------------------------------------
class procedure TXAMWindows.SetAppRun(Description, FileName : string; State, RunOnce : boolean);
var s : string;
begin
  with (TRegistry.Create) do
    try
      s := REGKEY_WINDOWS_RUN;
      if (RunOnce) then s := s + 'once';
      RootKey := HKEY_CURRENT_USER;
      OpenKey(s, true);
      case (State) of
        True  : WriteString(Description, FileName);
        False : DeleteValue(Description);
      end;
    finally
      Free;
    end;
end;

//-------------------------------------------------------------------------------------------------
// TXAMWindows.SetScreenSaverState
//-------------------------------------------------------------------------------------------------
class procedure TXAMWindows.SetScreenSaverState(Active : boolean);
begin
  Win32Check(SystemParametersInfo(SPI_SETSCREENSAVEACTIVE, integer(Active), nil, 0));
end;

//-------------------------------------------------------------------------------------------------
// TXAMWindows.SetSystemKeys
//-------------------------------------------------------------------------------------------------
class procedure TXAMWindows.SetSystemKeys(State : boolean); // Alt+Tab et Ctrl+Alt+Del
var Dummy : Integer;
begin
  dummy := application.MainForm.handle;
  Win32Check(SystemParametersInfo(SPI_SCREENSAVERRUNNING, word(not State), @Dummy, 0));
end;

//-------------------------------------------------------------------------------------------------
// TXAMWindows.SetSystemAltTabKeys
//-------------------------------------------------------------------------------------------------
class procedure TXAMWindows.SetSystemAltTabKeys(State : boolean); // Alt+Tab
var Dummy : Integer;
begin
  Win32Check(SystemParametersInfo(SPI_SETFASTTASKSWITCH, word(not State), @Dummy, 0));
end;

{
<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<->>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
                                                     TXAMApplication
<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<->>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
}

//-------------------------------------------------------------------------------------------------
// TXAMApplication.Create
//-------------------------------------------------------------------------------------------------
constructor TXAMApplication.Create;
var i      : integer;
    dbg    : cardinal;
begin
  try
    FLogFile        := TXAMLogFile.Create;
    FCmdLine        := TStringList.Create;
    FDelphiOwner    := Application.FindComponent('AppBuilder') <> nil;
    asm
      push eax
      push ecx
      mov eax, fs:[$18]               // Get the TIB's linear address
      mov eax, dword ptr [eax + $30]
      mov ecx, dword ptr [eax]
      mov [dbg], ecx
      pop ecx
      pop eax
    end;
    FRunByDebugger    := (dbg and $00010000 ) <> 0; // The 3rd byte is the byte to check for the presence of a debugger
    FDelphiRunning    := (FindWindow('TAlignPalette', nil) <> 0) and
                         (FindWindow('TPropertyInspector', nil) <> 0) and
                         (FindWindow('TAppBuilder', nil) <> 0);
    FFilename         := GetLongFileName(Application.ExeName);
    FPathBin          := IncludeTrailingBackslash(ExtractFilePath(FFilename));
    FPathRoot         := IncludeTrailingBackslash(ExtractFilePath(ExcludeTrailingBackslash(FPathBin)));
    FExt              := ExtractFileExt(FFilename);
    FVersion          := XAMGetFileVersion(FFileName);
    FTitle            := Application.Title;
    FTitleWithVer     := Application.Title + ' ' + FVersion;
    FState            := asInit;
    FLogFile.FileName := ChangeFileExt(FFileName, '.log');
    for i := 1 to ParamCount do FCmdLine.Add(UpperCase(ParamStr(i)));
    case (DelphiOwner) of
      True  : XAM_LOGFILE := false;
      False : case (CmdLine.IndexOf('/LOGFILE') <> -1) of
                  True  : XAM_LOGFILE := true;
                  False : if (CmdLine.IndexOf('/NOLOGFILE') <> -1) then XAM_LOGFILE := false;
              end;
    end;
  except
    if (not ModuleIsPackage) then raise;
  end;
end;

//-------------------------------------------------------------------------------------------------
// TXAMApplication.Destroy
//-------------------------------------------------------------------------------------------------
destructor TXAMApplication.Destroy;
begin
  FreeObject(FLogFile);
  FreeObject(FCmdLine);
  inherited;
end;

//-------------------------------------------------------------------------------------------------
// TXAMApplication.SetPriority
//-------------------------------------------------------------------------------------------------
// PURPOSE  : Fixe la priorit du processus.
//-------------------------------------------------------------------------------------------------
procedure TXAMApplication.SetPriority(Value : TXAMAppPriority);
begin
  Win32Check(SetPriorityClass(GetCurrentProcess, TAB_PRIORITY_CLASS_LIST[Value]));
end;

//-------------------------------------------------------------------------------------------------
// TXAMApplication.GetPriority
//-------------------------------------------------------------------------------------------------
// PURPOSE  : Dtermine la priorit du processus.
//-------------------------------------------------------------------------------------------------
function TXAMApplication.GetPriority : TXAMAppPriority;
var res : dword;
begin
  res := GetPriorityClass(GetCurrentProcess);
  if (res = 0) then RaiseLastWin32Error;
  for result := Low(TAB_PRIORITY_CLASS_LIST) to High(TAB_PRIORITY_CLASS_LIST) do
    if (TAB_PRIORITY_CLASS_LIST[result] = res) then Exit;
  result := pcNormal;
end;

//-------------------------------------------------------------------------------------------------
// TXAMApplication.ShowMessageLog
//-------------------------------------------------------------------------------------------------
procedure TXAMApplication.ShowMessageLog(Msg : string);
begin
  FLogFile.Write(logInfo, Msg);
  ShowMessage(Msg);
end;

//-------------------------------------------------------------------------------------------------
// TXAMApplication.TestOnlyOneInstance
//-------------------------------------------------------------------------------------------------
function TXAMApplication.TestOnlyOneInstance : boolean;
begin
  FMutex := CreateMutex(nil, false, PChar(Application.Title + ' Mutex For Only One Instance'));
  result := not ( (FMutex <> 0) and (GetLastError <> ERROR_ALREADY_EXISTS) );
  if (result) then CloseHandle(FMutex);
end;

{
#######################################################################################################################
                                                     INITIALIZATION
#######################################################################################################################
}

initialization
  try
    XAMInitializeSystem;
  except
    if (not ModuleIsPackage) then raise;
  end;

finalization
  FreeObject(XAMWindows);
  FreeObject(XAMApplication);

end.

